@@ -4,19 +4,20 @@ This module contains all classes and functions necessary for parsing a Instance
 This module will also access other Modules i.e TaxonomySchema.py to parse the Instance file
 as well as the taxonomies and linkbases used by the instance files
 """
-import re
 import abc
+import json
 import logging
-from io import StringIO, BytesIO
-from typing import List
+import re
 import xml.etree.ElementTree as ET
 from datetime import date, datetime
+from io import StringIO
+from typing import List
 
 from xbrl import TaxonomyNotFound, InstanceParseException
 from xbrl.cache import HttpCache
-from xbrl.taxonomy import Concept, TaxonomySchema, parse_taxonomy, parse_common_taxonomy, parse_taxonomy_url
 from xbrl.helper.uri_helper import resolve_uri
 from xbrl.helper.xml_parser import parse_file
+from xbrl.taxonomy import Concept, TaxonomySchema, parse_taxonomy, parse_common_taxonomy, parse_taxonomy_url
 from xbrl.transformations import normalize, TransformationException, TransformationNotImplemented
 
 logger = logging.getLogger(__name__)
@@ -172,20 +173,39 @@ class AbstractFact(abc.ABC):
     A Fact combines a Value with a Concept and a Context
     """
 
-    def __init__(self, concept: Concept, context: AbstractContext, value: any) -> None:
+    def __init__(self, concept: Concept, context: AbstractContext, value: any, xml_id: str or None) -> None:
         """
         :param concept: concept from the taxonomy, that the fact is referencing
         :param context: context of the fact
         :param value: value of the fact (can be number or text)
+        :param xml_id: id value of the fact
         """
         self.concept: Concept = concept
         self.context: AbstractContext = context
         self.value: any = value
+        self.xml_id = xml_id
         self.footnote: Footnote or None = None
 
     def __str__(self) -> str:
         return "{}: {}".format(self.concept.name, str(self.value))
 
+    def json(self, **kwargs) -> dict:
+        if isinstance(self.context, TimeFrameContext):
+            period: str = f"{self.context.start_date}/{self.context.end_date}"
+        elif isinstance(self.context, InstantContext):
+            period: str = str(self.context.instant_date)
+        else:
+            period: str = ''  # Forever context not specified in REC-2021-10-13
+
+        kwargs['value'] = self.value
+        if 'dimensions' not in kwargs: kwargs['dimensions'] = {}
+        kwargs['dimensions']['concept'] = self.concept.name
+        kwargs['dimensions']['entity'] = self.context.entity
+        kwargs['dimensions']['period'] = period
+        for segment in self.context.segments:
+            kwargs['dimensions'][segment.dimension.name] = segment.member.name
+        return kwargs
+
 
 class NumericFact(AbstractFact):
     """
@@ -196,7 +216,7 @@ class NumericFact(AbstractFact):
     """
 
     def __init__(self, concept: Concept, context: AbstractContext, value: float or None, unit: AbstractUnit,
-                 decimals: int or None) -> None:
+                 decimals: int or None, xml_id: str or None) -> None:
         """
         :param concept: see Abstract Fact
         :param context: see Abstract Fact
@@ -205,10 +225,13 @@ class NumericFact(AbstractFact):
         :param decimals: how accurate the number is. If decimals is none, the value of the fact is considered as
         accurate, without rounding errors
         """
-        super().__init__(concept, context, value)
+        super().__init__(concept, context, value, xml_id)
         self.unit: AbstractUnit = unit
         self.decimals: int or None = decimals
 
+    def json(self) -> dict:
+        return super().json(dimensions={"unit": str(self.unit)}, decimals=self.decimals)
+
 
 class TextFact(AbstractFact):
     """
@@ -218,8 +241,8 @@ class TextFact(AbstractFact):
     @warning The content of a Text fact can be huge. Especially for Text Blocks, those can also contain HTML code
     """
 
-    def __init__(self, concept: Concept, context: AbstractContext, value: str) -> None:
-        super().__init__(concept, context, value)
+    def __init__(self, concept: Concept, context: AbstractContext, value: str, xml_id: str or None) -> None:
+        super().__init__(concept, context, value, xml_id)
 
 
 class Footnote:
@@ -263,17 +286,44 @@ class XbrlInstance(abc.ABC):
         file_name: str = self.instance_url.split('/')[-1]
         return "{} with {} facts".format(file_name, len(self.facts))
 
+    def json(self, file_path: str = None, override_fact_ids: bool = True) -> str or None:
+        """
+        Converts the instance document into json format
+        :param file_path: if a path is given the function will store the json there
+        :param override_fact_ids:
+        :return: string (serialized json) or None (if file_path was given)
+
+        https://www.xbrl.org/Specification/xbrl-json/REC-2021-10-13/xbrl-json-REC-2021-10-13.html
+        """
+        json_dict: dict = {
+            "facts": {},
+            "documentInfo": {
+                "documentType": "https://xbrl.org/2021/xbrl-json",
+                "taxonomy": self.taxonomy.get_schema_urls(),
+                "baseUrl": self.instance_url
+            }
+        }
+        for i, fact in enumerate(self.facts):
+            fact_id = fact.xml_id if fact.xml_id and not override_fact_ids else f'f{i}'
+            json_dict['facts'][fact_id] = fact.json()
+        if file_path:
+            with open('data.json', 'w') as f:
+                return json.dump(json_dict, f)
+        else:
+            return json.dumps(json_dict)
+
 
 def parse_xbrl_url(instance_url: str, cache: HttpCache) -> XbrlInstance:
     """
-    Parses a instance file with it's taxonomy
+    Parses a instance file with it's taxonomy. This function will check, if the instance file is already
+    in the cache and load it from there based on the instance_url.
+    For EDGAR submissions: Before calling this method; extract the enclosure and copy the files to the cache.
+    i.e. Use CacheHelper.extract_edgar_enclosure()
+
     :param instance_url: url to the instance file (on the internet)
     :param cache: HttpCache instance
-    This function will check, if the instance file is already in the cache and load it from there based on the
-    instance_url.
-    For EDGAR submissions: Before calling this method; extract the enclosure and copy the files to the cache.
-        i.e. Use CacheHelper.extract_edgar_enclosure()
-    :return:
+
+    :return: parsed XbrlInstance object containing all facts with additional information
     """
     instance_path: str = cache.cache_file(instance_url)
     return parse_xbrl(instance_path, cache, instance_url)
@@ -282,12 +332,13 @@ def parse_xbrl_url(instance_url: str, cache: HttpCache) -> XbrlInstance:
 def parse_xbrl(instance_path: str, cache: HttpCache, instance_url: str or None = None) -> XbrlInstance:
     """
     Parses a instance file with it's taxonomy
+
     :param instance_path: url to the instance file (on the internet)
     :param cache: HttpCache instance
     :param instance_url: optional url to the instance file. Is sometimes necessary if the xbrl filings have their own
-    extension taxonomy. If i.e. a submission from the sec is parsed, the instance file might reference the taxonomy schema
-    with a relative path (since it is in the same directory as the instance file) schemaRef="./aapl-20211231.xsd"
-    :return:
+        extension taxonomy. If i.e. a submission from the sec is parsed, the instance file might reference the taxonomy schema
+        with a relative path (since it is in the same directory as the instance file) schemaRef="./aapl-20211231.xsd"
+    :return: parsed XbrlInstance object containing all facts with additional information
     """
     root: ET.Element = parse_file(instance_path).getroot()
     # get the link to the taxonomy schema and parse it
@@ -326,6 +377,8 @@ def parse_xbrl(instance_path: str, cache: HttpCache, instance_url: str or None =
         if fact_elem.text is None or len(str(fact_elem.text).strip()) == 0:
             continue
 
+        xml_id: str or None = fact_elem.attrib['id'] if 'id' in fact_elem.attrib else None
+
         # find the taxonomy where the tag is coming from
         taxonomy_ns, concept_name = fact_elem.tag.split('}')
         taxonomy_ns = taxonomy_ns.replace('{', '')
@@ -333,7 +386,12 @@ def parse_xbrl(instance_path: str, cache: HttpCache, instance_url: str or None =
         tax = taxonomy.get_taxonomy(taxonomy_ns)
         if tax is None: tax = _load_common_taxonomy(cache, taxonomy_ns, taxonomy)
 
-        concept: Concept = tax.concepts[tax.name_id_map[concept_name]]
+        try:
+            concept: Concept = tax.concepts[tax.name_id_map[concept_name]]
+        except KeyError:
+            logger.warning(f"Instance file uses invalid concept {concept_name}, thus fact {fact_elem.attrib['id']} "
+                           f"won't be parsed!")
+            continue
         context: AbstractContext = context_dir[fact_elem.attrib['contextRef'].strip()]
 
         if 'unitRef' in fact_elem.attrib:
@@ -342,10 +400,10 @@ def parse_xbrl(instance_path: str, cache: HttpCache, instance_url: str or None =
             unit: AbstractUnit = unit_dir[fact_elem.attrib['unitRef'].strip()]
             decimals_text: str = str(fact_elem.attrib['decimals']).strip()
             decimals: int = None if decimals_text.lower() == 'inf' else int(decimals_text)
-            fact = NumericFact(concept, context, float(fact_elem.text), unit, decimals)
+            fact = NumericFact(concept, context, float(fact_elem.text), unit, decimals, xml_id)
         else:
             # the fact is probably a text fact
-            fact = TextFact(concept, context, fact_elem.text.strip())
+            fact = TextFact(concept, context, fact_elem.text.strip(), xml_id)
         facts.append(fact)
 
     return XbrlInstance(instance_url if instance_url else instance_path, taxonomy, facts, context_dir, unit_dir)
@@ -354,29 +412,24 @@ def parse_xbrl(instance_path: str, cache: HttpCache, instance_url: str or None =
 def parse_ixbrl_url(instance_url: str, cache: HttpCache) -> XbrlInstance:
     """
     Parses a inline XBRL (iXBRL) instance file.
+
     :param cache: HttpCache instance
     :param instance_url: url to the instance file(on the internet)
-    This function will check, if the instance file is already in the cache and load it from there based on the
-    instance_url.
-    For EDGAR submissions: Before calling this method; extract the enclosure and copy the files to the cache.
-        i.e. Use CacheHelper.extract_edgar_enclosure()
-    :return:
+    :return: parsed XbrlInstance object containing all facts with additional information
     """
     instance_path: str = cache.cache_file(instance_url)
     return parse_ixbrl(instance_path, cache, instance_url)
 
 
-def parse_ixbrl(instance_path: str, cache: HttpCache, instance_url: str or None = None) -> XbrlInstance:
+def parse_ixbrl(instance_path: str, cache: HttpCache, instance_url: str or None = None, encoding=None) -> XbrlInstance:
     """
     Parses a inline XBRL (iXBRL) instance file.
+
     :param instance_path: path to the submission you want to parse
     :param cache: HttpCache instance
     :param instance_url: url to the instance file(on the internet)
-    This function will check, if the instance file is already in the cache and load it from there based on the
-    instance_url.
-    For EDGAR submissions: Before calling this method; extract the enclosure and copy the files to the cache.
-        i.e. Use CacheHelper.extract_edgar_enclosure()
-    :return:
+    :param encoding: optionally specify a file encoding
+    :return: parsed XbrlInstance object containing all facts with additional information
     """
     """
     In contrary to the XBRL-parse method we use here the actual root instead of the root element!!!
@@ -384,7 +437,7 @@ def parse_ixbrl(instance_path: str, cache: HttpCache, instance_url: str or None
     => in the XBRL-parse function root is ET.Element, here just an instance of ElementTree class!
     """
 
-    instance_file = open(instance_path, "r")
+    instance_file = open(instance_path, "r", encoding=encoding)
     contents = instance_file.read()
     pattern = r'<[ ]*script.*?\/[ ]*script[ ]*>'
     contents = re.sub(pattern, '', contents, flags=(re.IGNORECASE | re.MULTILINE | re.DOTALL))
@@ -427,6 +480,8 @@ def parse_ixbrl(instance_path: str, cache: HttpCache, instance_url: str or None
         tax = taxonomy.get_taxonomy(ns_map[taxonomy_prefix])
         if tax is None: tax = _load_common_taxonomy(cache, ns_map[taxonomy_prefix], taxonomy)
 
+        xml_id: str or None = fact_elem.attrib['id'] if 'id' in fact_elem.attrib else None
+
         concept: Concept = tax.concepts[tax.name_id_map[concept_name]]
         context: AbstractContext = context_dir[fact_elem.attrib['contextRef'].strip()]
         # ixbrl values are not normalized! They are formatted (i.e. 123,000,000)
@@ -438,10 +493,10 @@ def parse_ixbrl(instance_path: str, cache: HttpCache, instance_url: str or None
             decimals_text: str = str(fact_elem.attrib['decimals']).strip() if 'decimals' in fact_elem.attrib else '0'
             decimals: int = None if decimals_text.lower() == 'inf' else int(decimals_text)
 
-            facts.append(NumericFact(concept, context, fact_value, unit, decimals))
+            facts.append(NumericFact(concept, context, fact_value, unit, decimals, xml_id))
         elif fact_elem.tag == '{' + ns_map['ix'] + '}nonNumeric':
             fact_value: str = _extract_non_numeric_value(fact_elem)
-            facts.append(TextFact(concept, context, str(fact_value)))
+            facts.append(TextFact(concept, context, str(fact_value), xml_id))
 
     return XbrlInstance(instance_url if instance_url else instance_path, taxonomy, facts, context_dir, unit_dir)
 
@@ -643,41 +698,29 @@ def _load_common_taxonomy(cache: HttpCache, namespace: str, taxonomy: TaxonomySc
 class XbrlParser:
     """
     XbrlParser to make interaction easier.
-
     """
 
     def __init__(self, cache: HttpCache):
         self.cache = cache
 
-    def parse_instance(self, url: str) -> XbrlInstance:
+    def parse_instance(self, uri: str, instance_url: str or None = None) -> XbrlInstance:
         """
         Parses a xbrl instance (either xbrl or ixbrl)
-        :param url: url to the instance file.
-            i.e: https://www.sec.gov/Archives/edgar/data/320193/000032019320000096/aapl-20200926.htm
-        :return:
-        """
-        if url.split('.')[-1] == 'xml' or url.split('.')[-1] == 'xbrl':
-            return parse_xbrl_url(url, self.cache)
-        return parse_ixbrl_url(url, self.cache)
 
-    def parse_instance_locally(self, path: str, instance_url: str or None = None) -> XbrlInstance:
-        """
-        Parses a locally stored xbrl instance (either xbrl or ixbrl)
-        NOTE:
+        .. warning::
+
             If the instance document or extension taxonomy have relative imports the parser will also search for those
             files locally!
-            Example: your instance document is located at './data/aapl/2020/aapl-20200926.html' and the instance document
-            imports the taxonomy using a relative path '<schemaRef href="./aapl-20200926.xsd"/>' the parser will search
-            the document at "./data/aapl/2020/aapl-20200926.xsd".
 
-        :param path: the path to the instance document you want to parse
+        :param uri: url to the instance file.
+            i.e: https://www.sec.gov/Archives/edgar/data/320193/000032019320000096/aapl-20200926.
         :param instance_url: this parameter overrides the above described behaviour. If you also provide the url where the
-        instance document was downloaded, the parser can fetch relative imports using this base url
+            instance document was downloaded, the parser can fetch relative imports using this base url
         :return:
         """
-        if path.split('.')[-1] == 'xml' or path.split('.')[-1] == 'xbrl':
-            return parse_xbrl(path, self.cache, instance_url)
-        return parse_ixbrl(path, self.cache, instance_url)
+        if uri.split('.')[-1] == 'xml' or uri.split('.')[-1] == 'xbrl':
+            return parse_xbrl_url(uri, self.cache) if uri.startswith('http') else parse_xbrl(uri, self.cache, instance_url)
+        return parse_ixbrl_url(uri, self.cache) if uri.startswith('http') else parse_ixbrl(uri, self.cache, instance_url)
 
     def __str__(self) -> str:
         return 'XbrlParser with cache dir at {}'.format(self.cache.cache_dir)
