@@ -1,16 +1,11 @@
 """
 This module contains all classes and functions necessary for parsing Taxonomy schema files.
-
-A Taxonomy schema defines the reportable concepts and links the linkbases to
-describe the relationships between the concepts.
-Taxonomy schemas can import multiple different taxonomy schemas.
-
 """
 import logging
 import os
-from typing import List
 import xml.etree.ElementTree as ET
 from functools import lru_cache
+from typing import List
 from urllib.parse import unquote
 
 from xbrl import XbrlParseException, TaxonomyNotFound
@@ -171,6 +166,7 @@ ns_schema_map: dict = {
     "http://fasb.org/srt/2019-01-31": "http://xbrl.fasb.org/srt/2019/elts/srt-2019-01-31.xsd",
     "http://fasb.org/srt/2020-01-31": "http://xbrl.fasb.org/srt/2020/elts/srt-2020-01-31.xsd",
     "http://fasb.org/srt/2021-01-31": "http://xbrl.fasb.org/srt/2021/elts/srt-2021-01-31.xsd",
+    "http://fasb.org/srt/2022": "https://xbrl.fasb.org/srt/2022/elts/srt-2022.xsd",
     "http://fasb.org/srt-roles/2018-01-31": "http://xbrl.fasb.org/srt/2018/elts/srt-roles-2018-01-31.xsd",
     "http://fasb.org/srt-roles/2019-01-31": "http://xbrl.fasb.org/srt/2019/elts/srt-roles-2019-01-31.xsd",
     "http://fasb.org/srt-roles/2020-01-31": "http://xbrl.fasb.org/srt/2020/elts/srt-roles-2020-01-31.xsd",
@@ -190,6 +186,8 @@ ns_schema_map: dict = {
     "http://fasb.org/us-gaap/2019-01-31": "http://xbrl.fasb.org/us-gaap/2019/elts/us-gaap-2019-01-31.xsd",
     "http://fasb.org/us-gaap/2020-01-31": "http://xbrl.fasb.org/us-gaap/2020/elts/us-gaap-2020-01-31.xsd",
     "http://fasb.org/us-gaap/2021-01-31": "http://xbrl.fasb.org/us-gaap/2021/elts/us-gaap-2021-01-31.xsd",
+    "http://fasb.org/us-gaap/2022-01-31": "https://xbrl.fasb.org/us-gaap/2022/elts/us-gaap-2022.xsd",
+
     "http://fasb.org/us-roles/2011-01-31": "http://xbrl.fasb.org/us-gaap/2011/elts/us-roles-2011-01-31.xsd",
     "http://fasb.org/us-roles/2012-01-31": "http://xbrl.fasb.org/us-gaap/2012/elts/us-roles-2012-01-31.xsd",
     "http://fasb.org/us-roles/2013-01-31": "http://xbrl.fasb.org/us-gaap/2013/elts/us-roles-2013-01-31.xsd",
@@ -267,6 +265,7 @@ ns_schema_map: dict = {
     "http://xbrl.sec.gov/currency/2019-01-31": "https://xbrl.sec.gov/currency/2019/currency-2019-01-31.xsd",
     "http://xbrl.sec.gov/currency/2020-01-31": "https://xbrl.sec.gov/currency/2020/currency-2020-01-31.xsd",
     "http://xbrl.sec.gov/currency/2021": "https://xbrl.sec.gov/currency/2021/currency-2021.xsd",
+    "http://xbrl.sec.gov/currency/2022": "https://xbrl.sec.gov/currency/2022/currency-2022.xsd",
     "http://xbrl.sec.gov/dei/2011-01-31": "https://xbrl.sec.gov/dei/2011/dei-2011-01-31.xsd",
     "http://xbrl.sec.gov/dei/2012-01-31": "https://xbrl.sec.gov/dei/2012/dei-2012-01-31.xsd",
     "http://xbrl.sec.gov/dei/2013-01-31": "https://xbrl.sec.gov/dei/2013/dei-2013-01-31.xsd",
@@ -275,6 +274,8 @@ ns_schema_map: dict = {
     "http://xbrl.sec.gov/dei/2019-01-31": "https://xbrl.sec.gov/dei/2019/dei-2019-01-31.xsd",
     "http://xbrl.sec.gov/dei/2020-01-31": "https://xbrl.sec.gov/dei/2020/dei-2020-01-31.xsd",
     "http://xbrl.sec.gov/dei/2021": "https://xbrl.sec.gov/dei/2021/dei-2021.xsd",
+    "http://xbrl.sec.gov/dei/2022": "https://xbrl.sec.gov/dei/2022/dei-2022.xsd",
+    "http://xbrl.sec.gov/dei/2021q4": "https://xbrl.sec.gov/dei/2021q4/dei-2021q4.xsd",
     "http://xbrl.sec.gov/dei-def/2021": "https://xbrl.sec.gov/dei/2021/dei-2021_def.xsd",
     "http://xbrl.sec.gov/dei-entire/2021": "https://xbrl.sec.gov/dei/2021/dei-entire-2021.xsd",
     "http://xbrl.sec.gov/dei-ent-std/2019-01-31": "https://xbrl.sec.gov/dei/2019/dei-ent-std-2019-01-31.xsd",
@@ -303,6 +304,7 @@ ns_schema_map: dict = {
     "http://xbrl.sec.gov/naics/2011-01-31": "https://xbrl.sec.gov/naics/2011/naics-2011-01-31.xsd",
     "http://xbrl.sec.gov/naics/2017-01-31": "https://xbrl.sec.gov/naics/2017/naics-2017-01-31.xsd",
     "http://xbrl.sec.gov/naics/2021": "https://xbrl.sec.gov/naics/2021/naics-2021.xsd",
+    "http://xbrl.sec.gov/naics/2022": "https://xbrl.sec.gov/naics/2022/naics-2022.xsd",
     "http://xbrl.sec.gov/rr/2010-02-28": "https://xbrl.sec.gov/rr/2010/rr-2010-02-28.xsd",
     "http://xbrl.sec.gov/rr/2012-01-31": "https://xbrl.sec.gov/rr/2012/rr-2012-01-31.xsd",
     "http://xbrl.sec.gov/rr/2018-01-31": "https://xbrl.sec.gov/rr/2018/rr-2018-01-31.xsd",
@@ -327,7 +329,8 @@ ns_schema_map: dict = {
     "http://xbrl.sec.gov/sic/2021": "https://xbrl.sec.gov/sic/2021/sic-2021.xsd",
     "http://xbrl.sec.gov/stpr/2011-01-31": "https://xbrl.sec.gov/stpr/2011/stpr-2011-01-31.xsd",
     "http://xbrl.sec.gov/stpr/2018-01-31": "https://xbrl.sec.gov/stpr/2018/stpr-2018-01-31.xsd",
-    "http://xbrl.sec.gov/stpr/2021": "https://xbrl.sec.gov/stpr/2021/stpr-2021.xsd", # Replace draft taxonomy with official STPR 2021 one once it is released
+    "http://xbrl.sec.gov/stpr/2021": "https://xbrl.sec.gov/stpr/2021/stpr-2021.xsd",
+    # Replace draft taxonomy with official STPR 2021 one once it is released
     "http://xbrl.us/ar/2008-03-31": "http://xbrl.us/us-gaap/1.0/non-gaap/ar-2008-03-31.xsd",
     "http://xbrl.us/ar/2009-01-31": "http://taxonomies.xbrl.us/us-gaap/2009/non-gaap/ar-2009-01-31.xsd",
     "http://xbrl.us/country/2008-03-31": "http://xbrl.us/us-gaap/1.0/non-gaap/country-2008-03-31.xsd",
@@ -435,15 +438,10 @@ class TaxonomySchema:
     def __init__(self, schema_url: str, namespace: str):
         """
         The imports array stores an array of all Schemas that are imported.
-        The current Taxonomy Schema can override the extended schemas in the following way:
-        1. Addition of new concepts:
-            New concepts are added in this TaxonomySchema to extend the concepts declared in the base Taxonomy schemas
-        2. Addition of resources:
-            The Label Linkbase of this taxonomy can add new labels to existing concepts from the base taxonomy
-        3. Overriding of relationships:
-            All Linkbases of this taxonomy can override i.e the order of concepts in a definition linkbase
-        4. Overriding of resources:
-            The Label Linkbase of this taxonomy can override the labels of the base taxonomy!
+
+
+        :param schema_url:
+        :param namespace:
         """
         self.imports: List[TaxonomySchema] = []
         self.link_roles: List[ExtendedLinkRole] = []
@@ -469,8 +467,7 @@ class TaxonomySchema:
         Returns the taxonomy with the given namespace (if it is the current taxonomy, or if it is imported)
         If the taxonomy cannot be found, the function will return None
         :param url: can either be the namespace or the schema url
-        :return either a TaxonomySchema obj or None
-        :return:
+        :return: either a TaxonomySchema obj or None
         """
         if compare_uri(self.namespace, url) or compare_uri(self.schema_url, url):
             return self
@@ -481,6 +478,17 @@ class TaxonomySchema:
                 return result
         return None
 
+    def get_schema_urls(self) -> []:
+        """
+        Returns an array of all taxonomy urls that are used by this taxonomy
+        Also includes the schema url of this taxonomy
+        :return:
+        """
+        urls: [] = [self.schema_url]
+        for imported_tax in self.imports:
+            urls += imported_tax.get_schema_urls()
+        return list(set(urls))
+
 
 def parse_common_taxonomy(cache: HttpCache, namespace: str) -> TaxonomySchema or None:
     """
@@ -502,9 +510,10 @@ def parse_common_taxonomy(cache: HttpCache, namespace: str) -> TaxonomySchema or
 def parse_taxonomy_url(schema_url: str, cache: HttpCache) -> TaxonomySchema:
     """
     Parses a taxonomy schema file from the internet
-    :param schema_url:
-    :param cache:
-    :return:
+
+    :param schema_url: full link to the taxonomy schema
+    :param cache: :class:`xbrl.cache.HttpCache` instance
+    :return: parsed :class:`xbrl.taxonomy.TaxonomySchema` object
     """
     if not schema_url.startswith('http'): raise XbrlParseException(
         'This function only parses remotely saved taxonomies. Please use parse_taxonomy to parse local taxonomy schemas')
@@ -516,11 +525,12 @@ def parse_taxonomy_url(schema_url: str, cache: HttpCache) -> TaxonomySchema:
 def parse_taxonomy(schema_path: str, cache: HttpCache, schema_url: str or None = None) -> TaxonomySchema:
     """
     Parses a taxonomy schema file.
+
     :param schema_path: url to the schema (on the internet)
-    :param cache: HttpCache instance
+    :param cache: :class:`xbrl.cache.HttpCache` instance
     :param schema_url: if this url is set, the script will try to fetch additionally imported files such as linkbases or
-    imported schemas from the remote location. If this url is None, the script will try to find those resources locally.
-    :return:
+        imported schemas from the remote location. If this url is None, the script will try to find those resources locally.
+    :return: parsed :class:`xbrl.taxonomy.TaxonomySchema` object
     """
     if schema_path.startswith('http'): raise XbrlParseException(
         'This function only parses locally saved taxonomies. Please use parse_taxonomy_url to parse remote taxonomy schemas')
