@@ -23,6 +23,7 @@ Prefix: ixt-sec
 Namespace: http://www.sec.gov/inlineXBRL/transformation/2015-08-31
 """
 import re
+from math import floor
 
 from xbrl.transformations.text2num import text2num, replace_text_numbers
 
@@ -88,6 +89,86 @@ monthNorm = {
     'december': '12'
 }
 
+# exchange maps (https://www.sec.gov/info/edgar/specifications/edgarfm-vol2-v59.pdf page 203)
+exchangeNorm = {
+    'new york stock exchange': 'NYSE',
+    'nasdaq global select market': 'NASDAQ',
+    'nasdaq stock market': 'NASDAQ',
+    'box exchange': 'BOX',
+    'nasdaq bx': 'BX',
+    'cboe c2 exchange': 'C2',
+    'cboe exchange': 'CBOE',
+    'chicago stock exchange': 'CHX',
+    'cboe byx exchange': 'CboeBYX',
+    'cboe bzx exchange': 'CboeBZX',
+    'cboe edga exchange': 'CboeEDGA',
+    'cboe edgx exchange': 'CboeEDGX',
+    'nasdaq gemx': 'GEMX',
+    'investors exchange': 'IEX',
+    'nasdaq ise': 'ISE',
+    'miami international securities exchange': 'MIAX',
+    'nasdaq mrx': 'MRX',
+    'nyse american': 'NYSEAMER',
+    'nyse arca': 'NYSEArca',
+    'nyse national': 'NYSENAT',
+    'miax pearl': 'PEARL',
+    'nasdaq phlx': 'Phlx',
+}
+
+stateNormUS = {
+    'alabama': 'AL',
+    'alaska': 'AK',
+    'arizona': 'AZ',
+    'arkansas': 'AR',
+    'california': 'CA',
+    'colorado': 'CO',
+    'connecticut': 'CT',
+    'delaware': 'DE',
+    'florida': 'FL',
+    'georgia': 'GA',
+    'hawaii': 'HI',
+    'idaho': 'ID',
+    'illinois': 'IL',
+    'indiana': 'IN',
+    'iowa': 'IA',
+    'kansas': 'KS',
+    'kentucky': 'KY',
+    'louisiana': 'LA',
+    'maine': 'ME',
+    'maryland': 'MD',
+    'massachusetts': 'MA',
+    'michigan': 'MI',
+    'minnesota': 'MN',
+    'mississippi': 'MS',
+    'missouri': 'MO',
+    'montana': 'MT',
+    'nebraska': 'NE',
+    'nevada': 'NV',
+    'new hampshire': 'NH',
+    'new jersey': 'NJ',
+    'new mexico': 'NM',
+    'new york': 'NY',
+    'north carolina': 'NC',
+    'north dakota': 'ND',
+    'ohio': 'OH',
+    'oklahoma': 'OK',
+    'oregon': 'OR',
+    'pennsylvania': 'PA',
+    'rhode island': 'RI',
+    'south carolina': 'SC',
+    'south dakota': 'SD',
+    'tennessee': 'TN',
+    'texas': 'TX',
+    'utah': 'UT',
+    'vermont': 'VT',
+    'virginia': 'VA',
+    'washington': 'WA',
+    'washington dc': 'DC',
+    'west virginia': 'WV',
+    'wisconsin': 'WI',
+    'wyoming': 'WY',
+}
+
 
 def yearNorm(year: str) -> str:
     if len(year) == 4: return year
@@ -193,6 +274,26 @@ def numDotDecimal(arg: str) -> str:
 
 # region ixt-sec mappings
 
+def durYear(arg: str) -> str:
+    # -22.3456 -> -P22Y4M4D (Period: 22 Years, 4 Months, 4 Days)
+    pos: bool = float(arg) >= 0
+    yearDec: float = abs(float(arg))
+    fullYears: int = floor(abs(yearDec))
+    fullDays: int = round((yearDec - fullYears) * 365.25)
+    fullMonths: int = floor(fullDays / 30.437)
+    remainingDays: int = round(fullDays - fullMonths * 30.437)
+    return f"{'P' if pos else '-P'}{fullYears}Y{fullMonths}M{remainingDays}D"
+
+
+def durMonth(arg: str) -> str:
+    # 22.3456 -> P22M10D (Period: 22 Months, 10 Days)
+    pos: bool = float(arg) >= 0
+    monthDec: float = abs(float(arg))
+    fullMonths: int = floor(abs(monthDec))
+    remainingDays: int = round((monthDec - fullMonths) * 30)
+    return f"{'P' if pos else '-P'}{fullMonths}M{remainingDays}D"
+
+
 def durWordSen(arg: str) -> str:
     value = replace_text_numbers(arg)
     years, months, days = 0, 0, 0
@@ -226,6 +327,44 @@ def ballotBox(arg: str) -> str:
         raise TransformationException("Invalid input for ballotBox transformation rule")
 
 
+def exchNameEN(arg: str) -> str:
+    # "The New York Stock Exchange " -> "NYSE"
+    name = arg.lower().strip()
+    # remove any commas, points, e.t.c and "inc" or "llc"
+    name = re.sub(r'[^\w\s\d]|inc|llc|the', '', name)
+    # remove multiple spaces ("  ")
+    name = re.sub('r {2,}', ' ', name.strip())
+    if name.upper() in exchangeNorm.values():
+        return name.upper()
+    try:
+        return exchangeNorm[name]
+    except KeyError:
+        raise TransformationException(f'Unknown exchange "{arg}"')
+
+
+def stateNameEN(arg: str) -> str:
+    # "Kentucky" -> "KY"
+    name = arg.lower().strip()
+    # remove any commas, points e.t.c
+    name = re.sub(r'[^\w\s\d]', '', name.strip().lower())
+    try:
+        return stateNormUS[name]
+    except KeyError:
+        raise TransformationException(f'Unknown US State "{arg}"')
+
+
+def entityFilerCategoryEN(arg: str) -> str:
+    # large accelerated filer -> Large Accelerated Filer
+    arg = arg.lower().strip()
+    if arg == 'large accelerated filer':
+        return 'Large Accelerated Filer'
+    elif arg == 'accelerated filer':
+        return 'Accelerated Filer'
+    elif arg == 'non-accelerated filer':
+        return 'Non-accelerated Filer'
+    raise TransformationException(f'Unknown filer category "{arg}"')
+
+
 # endregion ixt-sec mappings
 
 ixt3 = {
@@ -361,8 +500,8 @@ ixt4 = {
 
 # As defined in https://www.sec.gov/info/edgar/specifications/edgarfm-vol2-v59.pdf
 ixt_sec = {
-    'duryear': notImplemented,
-    'durmonth': notImplemented,
+    'duryear': durYear,
+    'durmonth': durMonth,
     'durweek': notImplemented,
     'durday': notImplemented,
     'durhour': notImplemented,
@@ -370,11 +509,11 @@ ixt_sec = {
     'numwordsen': numWordSen,
     'datequarterend': notImplemented,
     'boolballotbox': ballotBox,
-    'exchnameen': notImplemented,
-    'stateprovnameen': notImplemented,
+    'exchnameen': exchNameEN,
+    'stateprovnameen': stateNameEN,
     'countrynameen': notImplemented,
     'edgarprovcountryen': notImplemented,
-    'entityfilercategoryen': notImplemented,
+    'entityfilercategoryen': entityFilerCategoryEN,
 }
 
 
